# Texel

Texel is a tool to help you build web applications, demos, creative coding sketches, data visualizations, and prototypes. It is inspired by Processing and other creative coding toolkits, but designed with modern JavaScript and web development in mind.

# About

Texel helps manage the "build step" part of modern web programming, but without any complex configuration or project setup. It attempts to simplify the following tasks:

- _Project Setup_ — a simple way to start a new project
- _Development_ – a fast and easy development experience
- _Bundling_ — a simple way to publish your web application

Texel handles things like reloading the browser when you edit your code, loading third-party dependencies, and transpiling modern JavaScript & CSS syntax for older browsers. See [Features](#features) for more details.

Unlike some other creative coding tools, like [Codepen.io](https://codepen.io/) and [Glitch](https://glitch.com/), Texel is designed for offline use and scalability: you can use it to build small sketches or a large production website.

# Quick Start

Texel comes in two flavours: Texel App and Texel CLI.

### Texel App

The Texel App makes it easy to jump right into creative coding without having to learn about modern JavaScript ecosystems like [Node.js](https://nodejs.org/en/) and [npm](http://npmjs.com/). This is great for beginners who just want to build a simple web page or experiment with creative coding.

The Texel App is also great for those running a code workshop, as it doesn't require your students to install Node.js or npm in order to take advantage of modern JavaScript development.

> [Quick Start: Texel App](./quick-start-texel-app.md)

## Texel CLI

If you are already familiar with [Node.js](https://nodejs.org/en/) and [npm](http://npmjs.com/), you may prefer to use Texel's command-line interface (CLI). This gives you greater control over Texel's features and options, and allows you to use your own version of Node.js and npm.

> [Quick Start: Texel CLI](./quick-start-texel-cli.md)

## Features

Texel comes with many features out of the box, including:

- Live reload browser on file save
- ES2015 syntax that transpiles down to ES5 (for older browsers)
- Ability to import/export files and grab libraries from `npm` (JavaScript bundling)
- CSS auto-prefixing and modern syntax support with [CSSNext](http://cssnext.io/)
- CSS pre-processor support (Post-CSS, SASS and LESS)
- GLSL shader reloading for WebGL applications
- Built-in support for React and JSX

For more details on how Texel works, see [Under the Hood](./under-the-hood.md).

## Docs

After you've gone through the quick start, you can browse through some other aspects of our documentation:

- Importing Dependencies
- Loading Files & Assets
- Publishing your first Website
- Project Examples:
  - ThreeJS (3D, WebGL)
  - D3 (data visualization)
  - P5.js
  - Canvas 2D
  - React
- Using Texel for Workshops
- Using Project Templates
- Live GLSL Reloading
- [Under the Hood](./under-the-hood.md)
