# texel

[![experimental](http://badges.github.io/stability-badges/dist/experimental.svg)](http://github.com/badges/stability-badges)


TODO:

- Auto pragma JSX for react vs preact/deku/hyperapp and other modules.
- Auto SASS / LESS (no install)? or have user install texel-css-to-sass and detect it? Latter could be done with templates
- "Sketch" - a simple template that you can define (i.e. "Save as Sketch Template")
- "Project" - choose from a set of templates, like:
  - website
  - ghpages
  - module
  - three-js
  - surge-sh
- How to handle local vs Electron vs global Texel versioning

## Shader ideas

Custom glslify + shader transform. It re-writes for shader-reload under the hood and eventually maybe we will re-write that too within texel (but for now can continue using shader-reload). Syntax:

```js
import shader from 'texel/util/shader';
import glslify from 'glslify';

export default shader({
  vertex: glslify('./file.vert'),
  fragment: glslify('./file.frag')
});
```

Or:

```js
import shader from 'texel/util/shader';
import glslify from 'glslify';

export default shader({
  vertex: `void main () {}`
  fragment: `void main () {}`
});
```

Notes:

- Fix glslify import/export syntax
- Fix glslify single line turning into a file
- Fix glslify lack of path.resolve
- Support path.resolve

OR:

Stick entirely with browserify syntax in case user wants to eject out of texel eventually?

OR:

```js
require('texel/shader')({ vertexFile, fragmentFile });
```

## Sketchbook

One sketchbook (project) can hold many sketches (JS files).

- Open App
- Create a Sketchbook


- Module loading like `import('texel/sketch').then()` ?
  - Deal with factor-bundle

- STRATEGIES
FACTOR-BUNDLE
  - One bundle.js for the "main" app (i.e. canvas container)
  - A separate bundle for each sketch
  - One bundle-common.js for the common features across all sketches & main app

SINGLE BUNDLE
  - One bundle.js that contains the "main" app and all sketches

```sh
texel sketch sketches/lines.js --book penplot
texel sketch 
```

- A way to specify a list of "sketches" (e.g. "lines.js", "colors.js")
  - Should allow ordering by user
  - Should allow entire folders (no need to re-order each time)
- Should be useful for real production apps, i.e. each sketch is a different page
- Should be generic, e.g. each sketch could be a Preact component for a blog
- Should be dynamic. Main sketchbook is loaded, then each sketch is loaded on demand

STEPS:
- User specifies "include patterns" (list of globs or list of files) like in a sketchbook.json file or something. Each file is prepared (lazily bundled on demand) and those folders are watched for new files that match the pattern.
- The app/framework/template handles the frontend and lazily/dynamically imports modules based on a string from URL hash or real URL (support pushstate and non-pushstate)


USER FLOW:

1. Create a new Sketchbook:
  - Must always be created from a template (e.g. Canvas 2D animation, Penplots, Preact components, GLSL sketches, etc)
  - There are templates for "bare" projects
  - What is the default template?

2. Immediately be presented with an example sketch.

3. Start sketching.

4. Finish sketch, start a new one by "Adding a new file" (either GUI, via tool, or manually via OS). New file should be able to be templated for convenience.

5. Once you are happy with all sketches, "Deploy" to create a static site ZIP.

GOALS:
- Should work entirely in localhost:9966 browser
- Should support regular budo style sites
- Should support an "index" with listing to all sketches
- Should support a custom "index" with fancy animations between sketches and however user wants



QUESTION: Pushstate by default (Netlify etc) or not (gh-pages, Surge)...?

- Specify a single folder that holds all your sketch files:
  --sketch src/sketches
- Then:
  - Sketch util API is generated (has all current sketches & folder name)
    - loadSketch
    - getSketchFolder
    - getSketchFiles
  - Mode: SPLIT (default)
    - loadSketch('foo.js')
    - This will actually load a `<script>` tag for public/sketches/fooba.js
  - Mode: STANDALONE
    - The generated sketch module requires all of them into a cache
    - The loadSketch('foo.js') resolves immediately since it just picks out of cache


single file vs multi file

SINGLE FILE: 
behave like budo / browserify
single bundle.js

MULTI FILE:
show an index into different files

## Install

Use [npm](https://npmjs.com/) to install.

```sh
npm install texel --save
```

## Usage

[![NPM](https://nodei.co/npm/texel.png)](https://www.npmjs.com/package/texel)

## License

MIT, see [LICENSE.md](http://github.com/mattdesl/texel/blob/master/LICENSE.md) for details.


texel create
  Scaffold a new project

  texel create
  texel create --template=mattdesl/texel-template-shader

texel (AKA texel start)
  Start a development server that looks for `public/` folder.
  This also "cleans" the output by removing the bundled style.css and bundle.js files.

  texel start
  texel start .
  texel start index.js
  texel start index.js --out main.js
  texel start index.js --dir .
  texel start index.js --dir foo/ --no-css
  texel start index.js --css style.css

texel bundle
  Build a static site

  texel bundle
  texel bundle .
  texel bundle index.js --out main.js
  texel bundle index.js --out index.js --dir . # throws error
  texel bundle index.js --dir .
  texel bundle --no-compress
  texel bundle --inline-style

SUPPORT:
  require('foo.less')
  require('main.css')
  require('bar.scss')


GOAL:
require('./foo.css')
  - Parsed/transformed CSS is appended to a root style sheet.
  - JS is transformed into:
  /* require('./foo.css') */
  - On bundle wrap, 

str = require('texel-transform-css')(file)
  - If .css file, use postcss + cssnext + autoprefixer
  - If .less file, resolve 'texel-transform-less' compiler from cwd
  - If .scss or .sass, resolve 'texel-transform-sass' compiler from cwd



texel start src/index.js --dir public/ --js bundle.js --css main.css
  - Serve public/
  - External CSS file (for LiveReload)

texel start src/index.js --js bundle.js --css main.css
  - Serve cwd
  - External CSS file

texel start src/index.js
  - Serve CWD
  - External CSS, default 'bundle.js' and 'main.css' files

texel bundle src/index.js
  - Write to CWD
  - External CSS JS, default 'bundle.js' and 'main.css'


entries ->
result = {
  bundles: [ { code: '..', name: 'bundle.js' } ],
  css: { code: '...', map: '...', dependencies: [] } 
}

texel bundle src/index.js
-> public/bundle.js
-> public/bundle.css
-> public/index.html (once)


texel start -> single input, single output
texel sketch -> multiple inputs, multiple outputs


texel start src/index.js (regular HTML site)
texel start src/lines.js --sketch (a Canvas, Print, or WebGL thing)
  alias: texel sketch src/lines.js

What if root of app exports a 'texel' settings object, and `texel start index.js` will pick up on that?

texel start cool.js
> Write an empty JS file at 'cool.js', you will create your own app from scratch.

texel start cool.js --sketch
texel sketch cool.js
> Writes a basic 2D canvas sketch at 'cool.js', you just need to add the drawing.

texel bundle cool.js
> Bundles the file into a static HTML site.

texel bundle sketches/*.js --all
> Bundles all sketches into separate html files?
> How to handle SPA type stuff?





texel start sketches/*.js -p 2d
texel start sketches/*.js -e 2d
texel start sketches/feb-26.js -a 2d

texel start foo.js --preset regl

PRESETS

2d / webgl
- Bare `<canvas>` preset

regl
- Extends webgl preset, adds latest 'regl' dependency
- Includes an import statement already

three
- Extends webgl preset, adds latest 'three' dependency
- Includes a simple WebGL scene

p5

d3

--

APP UX

- CREATE NEW SKETCH
  - Allow sketch to be created from a preset?
- OPEN RECENT SKETCH

CREATE -> Choose folder & name

DEV -> You see project 