var stream = require('stream');
var babel = require('@babel/core');
var util = require('util');
var path = require('path');

module.exports = Babelify;
util.inherits(Babelify, stream.Transform);

function Babelify (filename, opts, filter) {
  if (!(this instanceof Babelify)) {
    return Babelify.configure(opts)(filename);
  }

  stream.Transform.call(this);
  this._data = '';
  this._filename = filename;
  this._filter = filter || (() => true);
  this._opts = Object.assign({filename: filename}, opts);
}

Babelify.prototype._transform = function (buf, enc, callback) {
  this._data += buf;
  callback();
};

Babelify.prototype._flush = function (callback) {
  const shouldFilter = this._filter(this._data, this._filename, this._opts);
  if (!shouldFilter) {
    this.push(this._data);
    process.nextTick(callback);
    return;
  }
  this.emit('babelify-start', this._filename);
  babel.transform(this._data, this._opts, (err, result) => {
    if (err) {
      this.emit('error', err);
    } else {
      this.emit('babelify-finish', this._filename, result);
      this.emit('babelify', result, this._filename); // backwards compatible
      if (!result) {
        this.emit('error', new Error(`Result object is ${result}`));
        return callback();
      }
      var code = result.code;
      this.push(code);
    }
    callback();
  });
};

Babelify.configure = function (opts) {
  opts = Object.assign({}, opts);
  var extensions = opts.extensions || babel.DEFAULT_EXTENSIONS;
  var sourceMapsAbsolute = opts.sourceMapsAbsolute;
  if (opts.sourceMaps !== false) opts.sourceMaps = 'inline';

  // babelify specific options
  delete opts.sourceMapsAbsolute;
  delete opts.extensions;
  delete opts.filename;

  // babelify backwards-compat
  delete opts.sourceMapRelative;

  // browserify specific options
  delete opts._flags;
  delete opts.basedir;
  delete opts.global;

  // browserify cli options
  delete opts._;
  // "--opt [ a b ]" and "--opt a --opt b" are allowed:
  if (opts.ignore && opts.ignore._) opts.ignore = opts.ignore._;
  if (opts.only && opts.only._) opts.only = opts.only._;
  if (opts.plugins && opts.plugins._) opts.plugins = opts.plugins._;
  if (opts.presets && opts.presets._) opts.presets = opts.presets._;

  const filter = opts.filter || (() => true);
  const passThroughFilter = opts.passThroughFilter || (() => true);
  delete opts.passThroughFilter;
  delete opts.filter;

  return function (filename, tOpts) {
    var extname = path.extname(filename);
    if (extensions.indexOf(extname) === -1) {
      return stream.PassThrough();
    }

    const shouldPassThrough = passThroughFilter(filename, tOpts);
    if (!shouldPassThrough) {
      return stream.PassThrough();
    }

    // Why is this even necessary?? babelify is running on all node modules
    // somehow??
    if (!tOpts.global) {
      const dir = path.dirname(filename);
      const parts = path.relative(opts.cwd, dir).split(path.sep);
      if (parts[0] === 'node_modules') {
        if (parts[1] !== 'texel') {
          // console.log('skip', path.relative(opts.cwd, filename));
          return stream.PassThrough();
        } else {
          // console.log('transform dep', path.relative(opts.cwd, filename));
        }
      }
    }

    var _opts = Object.assign({
      sourceFileName: sourceMapsAbsolute ? filename : path.basename(filename)
    }, opts);

    return new Babelify(filename, _opts, filter);
  };
};
