const path = require('path');
const resolve = require('resolve');

module.exports = function ({ template, types: t }) {
  return {
    visitor: {
      ExportNamedDeclaration: {
        exit (nodePath, params) {
          const { opts } = params;
          const filename = params.filename;
          if (typeof opts.filter === 'function' && !opts.filter(filename)) {
            return;
          }

          const names = Object.keys(opts.exports);
          if (names.length === 0) return;

          const specifiers = nodePath.get('specifiers');
          if (specifiers && specifiers.length > 0) {
            const specs = specifiers.filter(spec => names.includes(spec.node.exported.name));
            specs.forEach(spec => {
              const binding = nodePath.scope.getBinding(spec.node.local.name);
              const key = spec.node.exported.name;
              if (!binding) return;
              if (binding && !binding.constant) {
                opts.exports[key] = { confident: false, constant: false, value: undefined };
                return;
              }
              if (binding.path.type === 'VariableDeclarator') {
                opts.exports[key] = evaluate(filename, binding.path.get('init'));
              }
            });
          } else {
            const declaration = nodePath.get('declaration');
            if (!declaration || !declaration.type) return;
            let declarations = declaration.get('declarations');
            if (!declarations || !declarations.length === 0 || !declarations.filter) return;
            declarations = declarations.filter(d => names.includes(d.node.id.name));
            if (declarations.length === 0) return;
            declarations.forEach(d => {
              opts.exports[d.node.id.name] = evaluate(filename, d.get('init'));
            });
          }
        }
      }
    }
  };

  function evaluate (file, init) {
    if (init && init.evaluate) {
      const result = init.evaluate();
      return { confident: result.confident, value: result.value };
    } else {
      return { confident: false, value: undefined };
    }
  }
};
