const path = require('path');
const ensureRequire = require('./ensure-require');

module.exports = function (body, file, opt) {
  return ensureRequire('node-sass', 'SASS', opt)
    .then(sassPath => {
      return new Promise((resolve, reject) => {
        const sass = require(sassPath);

        // provide import path
        const paths = [ path.dirname(file), opt.basedir ];

        const sassOpts = {
          outputStyle: opt.compress ? 'compressed' : undefined,
          includePaths: paths,
          sourceComments: true,
          data: body,
          file: file
        };

        return sass.render(sassOpts, function (err, res) {
          if (err) {
            reject(err);
          } else {
            res.stats.includedFiles.forEach(dep => {
              if (dep !== file) opt.compiler.emit('file', dep);
            });
            resolve(res.css.toString());
          }
        });
      });
    })
    .then(result => {
      if (typeof result === 'object') {
        result.imports.forEach(dep => {
          dep = path.isAbsolute(dep) ? dep : path.resolve(path.dirname(file), dep);
          opt.compiler.emit('file', dep);
        });
        result = result.css;
      }
      return result;
    });
};
