const path = require('path');
const { SimpleError } = require('../util/logger');

module.exports = function (bundler, pluginOpts = {}) {
  const basedir = path.resolve(__dirname, '../../');
  const resolver = bundler._bresolve;
  const modules = [ 'glslify', 'shader-reload', 'texel/util' ];
  let isDev = process.env.NODE_ENV === 'development';

  if (typeof bundler._bresolve !== 'function') {
    throw new Error('ERROR: It looks like somehow Texel is using an unsupported version ' +
        'of browserify, please file a bug report in Texel GitHub Issues:\n    https://github.com/mattdesl/texel/issues');
  }

  // Resolve certain modules from here instead of user working dir
  bundler._bresolve = function (id, opts, cb) {
    if (id === 'texel/devtool') {
      throw new SimpleError(`In file ${path.relative(process.cwd(), opts.filename)}\n  "texel/devtool" module has been renamed to "texel/util/exporter"`);
    }
    if (modules.some(m => id.startsWith(m))) {
      opts = Object.assign({}, opts, { basedir });

      // Special resolve for dev tools which are optimized away in bundle mode
      if (id === 'texel/util/exporter') {
        if (isDev) id = './util/exporter/development.js';
        else id = './util/exporter/production.js';
      }

      // Relocate requires to the correct (running) version of texel
      id = id.replace('texel/util', './util');
    }
    return resolver.call(bundler, id, opts, (err, result, pkg, foo) => {
      // Improve error messaging since browserify sometimes gives you just a folder,
      // not the actual file it was required by. Could improve further by parsing
      // file and getting real syntax error message.
      if (err) {
        cb(new SimpleError(`Cannot find module '${id}' from '${path.relative(path.dirname(process.cwd()), opts.filename)}'`));
      } else {
        cb(null, result, pkg);
      }
    });
  };
};
