const maxstache = require('maxstache');
const through = require('through2');
const fs = require('fs');
const path = require('path');
const fromString = require('from2-string');

module.exports = function (bundler, opt = {}) {
  const cwd = opt.cwd;
  const sketchFolder = path.isAbsolute(opt.sketches) ? opt.sketches : path.resolve(cwd, opt.sketches);
  const importModule = 'texel/util/sketches';
  const apiFile = require.resolve('./sketch-api');

  // bundler.exclude(importModule);
  // bundler.add(apiFile, {
  //   file: 'texel-generated-sketch-loader.js',
  //   expose: importModule
  // });

  let firstRecord = true;
  bundler.pipeline.get('record').unshift(through.obj(function (chunk, enc, next) {
    if (firstRecord) {
      firstRecord = false;
      this.push({
        transform: transform,
        global: undefined
      });
      next(null, chunk);
    } else {
      next(null, chunk);
    }
  }));

  // This is fairly annoying but there is no way to manually "invalidate"
  // a generated module (that I have yet found), so it sits around in watchify cache.
  process.nextTick(() => {
    const createWatcher = bundler._watcher;
    if (typeof bundler._watcher !== 'function') {
      throw new Error('Invalid watchify version; this may represent a bug with texel. Please file a new issue in our GitHub.');
    }
    bundler._watcher = function (dep, wopts) {
      return createWatcher.call(bundler, dep, wopts);
    };
  });

  return {
    addSketchFile (file) {
      bundler.emit('file', file);
    }
  };

  function generateCache (files) {
    const modules = files.map((file, i) => {
      let absFile = path.resolve(sketchFolder, file);
      const end = i === files.length - 1 ? '' : ',';

      const stat = fs.statSync(absFile);
      if (stat.isDirectory()) {
        // add index.js to directory for components
        file = path.join(file, 'index.js');
        absFile = path.join(absFile, 'index.js');
      }

      return `  ${JSON.stringify(file)}: require(${JSON.stringify(absFile)})${end}`;
    }).join('\n');
    return `{\n${modules}\n}`;
  }

  function transform (file, tOpts) {
    if (file === apiFile) {
      let code = '';
      const write = (chunk, enc, next) => {
        code += chunk.toString();
        next(null, '');
      };
      const end = function (next) {
        let data;
        try {
          const files = fs.readdirSync(sketchFolder);

          // Generate cache may fail on sync dir tests
          data = {
            files: JSON.stringify(files),
            folder: JSON.stringify(path.relative(cwd, sketchFolder)),
            cache: generateCache(files)
          };
        } catch (err) {
          this.emit('error', err);
          next();
          return;
        }

        const result = maxstache(code, data);
        this.push(result);
        next(null);
      };

      const out = through(write, end);
      out.emit('file', apiFile);
      return out;
    } else {
      return through();
    }
  }
};
