const cache = {{cache}};
const files = {{files}};
const folder = {{folder}};

module.exports = {
  loadSketch,
  getSketchFiles: () => files,
  getSketchFolder: () => folder
};

function loadSketch (fileName) {
  return new Promise((resolve, reject) => {
    let key = resolveFileName(fileName);

    // First resolve 'bar' to a directory with index, i.e. 'bar/index.js'
    if (!(key in cache) && !/\/$/.test(key)) {
      const newKey = key + '/index.js';
      if (newKey in cache) key = newKey;
    }

    // If that doesn't work, resolve 'bar' to 'bar.js'
    if (!(key in cache) && !/\./.test(key)) {
      const newKey = key + '.js';
      if (newKey in cache) key = newKey;
    }

    // If that doesn't work, user must not have put in correct file name
    if (!(key in cache)) {
      return reject(new Error(`Cannot find a sketch in ${folder} by the file name ${fileName}`));
    }

    return resolve(cache[key]);
  });
}

function resolveFileName (name) {
  // Strip starting './' and '/'
  name = name.replace(/^\.\//, '').replace(/^\//, '');
  // If ends with '/', add 'index.js' to end
  if (/\/$/.test(name)) name = name + 'index.js';
  return name;
}
