/** @jsx h */
import { h, Component } from 'preact';
import Canvas from './Canvas';

const isDev = process.env.NODE_ENV !== 'production';

export default class App extends Component {
  constructor () {
    super();
    this.state = {
      recording: false,
      totalFrames: 0,
      frame: 0
    };
  }

  componentDidMount () {
    if (isDev) {
      window.addEventListener('keydown', ev => {
        const isRecording = this.state.recording;
        if (ev.keyCode === 32) {
          this.setState({ recording: isRecording ? false : 'gif' });
        } else if (ev.keyCode === 82) {
          if (ev.metaKey || ev.shiftKey || ev.ctrlKey) return;
          this.setState({ recording: isRecording ? false : 'frames' });
        }
      });
    }
  }

  handleRecordFinish = () => {
    this.setState({ recording: false });
  }

  handleRecordProgress = ({ frame, totalFrames }) => {
    this.setState({ frame, totalFrames });
  }

  render ({ scene, config }) {
    let caption;
    if (isDev) {
      caption = this.state.recording
          ? <caption className='recording'>
            Recording {this.state.recording === 'gif' ? 'GIF' : 'Frames'} {
              `(${this.state.frame + 1} / ${this.state.totalFrames})`
            }
          </caption>
          : <caption>
              <strong>SPACE</strong> to Record GIF - <strong>R</strong> to record PNG sequence
            </caption>
    } else {
      caption = <caption>
        <div class="name">{config.name}</div><div class="byline">by <a href={config.url}>{config.author}</a></div>
      </caption>
    }
    return <main>
      <Canvas
        {...config}
        scene={scene}
        onRecordFinish={this.handleRecordFinish}
        onRecordProgress={this.handleRecordProgress}
        recording={this.state.recording}
      />
      {
        caption
      }
    </main>;
  }
}
