const format = require('cli-format');
const chalk = require('chalk');

const wrapOpts = { paddingLeft: '  ', paddingRight: '  ' };

module.exports.info = function (str) {
  str = format.wrap(str, wrapOpts);
  return `\n${str}\n`;
};

module.exports.log = function (message) {
  return chalk.green(`→ ${message}`);
};

module.exports.error = function (err) {
  const result = err.isSimpleError
    ? format.wrap(`${chalk.red('ERROR:')} ${err.message}`, wrapOpts)
    : err.stack;
  return `\n${result}\n`;
};

// If an error is "simple" (i.e. clear user accident) then don't blast console
// with long stack traces. Generally trying to keep the dev experience clean
// and simple for new-comers.
module.exports.SimpleError = class SimpleError extends Error {
  constructor (message, filename, lineNumber) {
    super(message, filename, lineNumber);
    this.isSimpleError = true;
  }
};
