const path = require('path');
const resolve = require('resolve');
const cwd = process.cwd();
const chalk = require('chalk');
const fs = require('fs');
const { SimpleError } = require('../util/logger');
const defaultEntry = 'index.js';

module.exports = resolveEntry;
function resolveEntry (file) {
  let entry = file;
  let missingMain = false;
  if (!entry) entry = '.';

  if (!path.isAbsolute(entry) && !/^\./.test(entry)) {
    entry = `./${entry}`;
  }

  if (entry === '.') {
    try {
      // attempt to resolve package.json in current directory
      const pkg = require(path.resolve(cwd, 'package.json'));
      if (!pkg.main) missingMain = true;
    } catch (err) {
      // Cannot find package, do some useful debug messages...
      entry = path.resolve(cwd, defaultEntry);
      if (!fs.existsSync(entry)) {
        throw new SimpleError(`No ${chalk.bold('package.json')} found in current directory, and no ${chalk.bold(defaultEntry)} file exists.\n\nTry specifying the entry file for your JavaScript code, like so:\n  texel [command] path/to/entry.js`);
      }
      return entry;
    }
  }

  try {
    return resolve.sync(entry, { basedir: cwd });
  } catch (err) {
    if (missingMain) {
      throw new SimpleError(`Cannot find an entry point ${chalk.bold(entry)}\nPoint to a JavaScript file, or include a "main" field in your ${chalk.bold('package.json')}. Example:\n  texel [command] path/to/entry.js`);
    } else if (!file) {
      throw new SimpleError(`Cannot find an entry point in folder ${chalk.bold(path.basename(cwd))}\nMake sure your "main" field in ${chalk.bold('package.json')} points to the right file, or change the texel script to point to a specific entry file. Example:\n  texel [command] path/to/entry.js`);
    } else {
      throw new SimpleError(`Cannot find the entry point ${chalk.bold(entry)}\nAre you sure the file exists in this directory?`);
    }
  }
}
