const spawn = require('cross-spawn');
const terminate = require('terminate');
const psTree = require('ps-tree');
const exitSignal = require('signal-exit');

let isKilled = false;
let pids = [];

function pop (pid) {
  const idx = pids.indexOf(pid);
  if (idx >= 0) pids.splice(idx, 1);
}

function kill (pid, signal) {
  return new Promise((resolve) => {
    terminate(pid, signal, (err) => {
      if (err) {
        console.warn('Could not kill PID', pid);
      }
      resolve();
    });
  });
}

module.exports = function (command, args = [], opt = {}) {
  return new Promise((resolve, reject) => {
    if (isKilled) return Promise.resolve();
    opt = Object.assign({}, opt);

    const child = spawn(command, args, opt);
    setTimeout(() => {
      psTree(child.pid, (err, tree) => {
        if (err) console.error(err);
        else {
          const newPids = tree.map(t => parseInt(t.PID, 10));
          newPids.forEach(p => {
            if (pids.indexOf(p) === -1) pids.push(p);
          });
        }
      });
    }, 1000);
    pids.push(child.pid);
    child.once('error', reject);
    child.once('close', (code, signal) => {
      pop(child.pid);
      if (code === 0 || isKilled) resolve();
      else reject(new Error(`Command ${args.slice(1, 2).filter(Boolean).join(' ')} closed with exit code ${code} and signal ${signal}`));
    });
  });
};

function killAll (signal = 'SIGTERM') {
  isKilled = true;
  return Promise.all(pids.map(p => kill(p, signal)));
}

module.exports.killAllSubprocesses = () => {
  return killAll();
};

exitSignal(() => {
  module.exports.killAllSubprocesses();
});
