const fs = require('fs');
const path = require('path');

function statAsync (dir) {
  return new Promise((resolve, reject) => fs.stat(dir, (err, stat) => {
    if (err) reject(err);
    else resolve(stat);
  }));
}

function readdirAsync (dir) {
  return new Promise((resolve, reject) => fs.readdir(dir, (err, files) => {
    if (err) reject(err);
    else resolve(files);
  }));
}

module.exports = function walker (config, fn) {
  const input = config.path;
  const filterFolders = config.filterFolders || (() => true);
  const filterFiles = config.filterFiles || (() => true);

  const processFile = (directory, filename, folders) => {
    const filePath = path.join(directory, filename);
    const fullPath = path.join(input, filePath);
    return statAsync(fullPath).then(stat => {
      const isDir = stat.isDirectory();
      const filter = isDir ? filterFolders : filterFiles;
      const params = {
        root: input,
        path: filePath,
        fullPath: fullPath,
        name: filename,
        directory: isDir,
        stat: stat
      };
      if (!filter(params)) return;
      if (isDir) folders.push(filePath);
      return fn(params);
    });
  };

  const walk = directory => {
    const folders = [];
    return readdirAsync(path.join(input, directory))
      .then(files => {
        return Promise.all(files.map(file => {
          return processFile(directory, file, folders);
        }));
      })
      .then(() => {
        return Promise.all(folders.map(folder => walk(folder)));
      });
  };

  return walk('');
};
